      subroutine dfcn(x,f,g,nprob)
      integer nprob
      double precision x, f, g
c     **********
c
c     Subroutine dfcn
c
c     This subroutine defines the test functions in the report
c
c     J. J. More' and D. J. Thuente,
c     Line search algorithms with guaranteed sufficient decrease,
c     ACM Transactions on  Mathematical Software, 20, (1994), 286--307
c
c     The subroutine statement is
c
c       subroutine dfcn(x,f,g,nprob)
c
c     where
c
c       x is a double precision variable.
c         On entry x specifies the point of evaluation.
c         On exit x is unchanged.
c
c       f is a double precision variable.
c         On entry f need not be specified.
c         On exit f contains the value of the function at x.
c
c       g is a double precision variable.
c         On entry g need not be specified.
c         On exit g contains the derivative of the function at x.
c
c       nprob is an integer.
c         On entry nprob specifies the number of the problem.
c            For this set 1 <= nprob <= 6.
c         On exit nprob is unchanged.
c
c     MINPACK-2 Project. March 1996.
c     Argonne National Laboratory.
c     Jorge J. More'.
c
c     **********
      double precision beta1, beta2, beta3
      parameter (beta1=2.0d0,beta2=4.0d-3,beta3=1.0d-2)
      double precision one, two
      parameter (one=1.0d0,two=2.0d0)

      integer nwig
      double precision a1, a2, t, t1, t2

c     Selection of test problems.

      if (nprob .eq. 1) then
         f = -x/(x**2+beta1)
         g = (x**2-beta1)/(x**2+beta1)**2
      else if (nprob .eq. 2) then
         t = x + beta2
         f = t**5 - two*t**4
         g = 5*t**4 - 4*two*t**3
      else if (nprob .eq. 3) then
         nwig = 39
         if (x .lt. one-beta3) then
            f = one - x
            g = -one
         else if (x .lt. one+beta3) then
            f = (x-one)**2/(two*beta3) + beta3/two
            g = (x-one)/beta3
         else
            f = x - one
            g = one
         end if
         t1 = two*nwig*atan(one)
         t2 = (one-beta3)/t1
         f = f + t2*sin(t1*x)
         g = g + (one-beta3)*cos(t1*x)
      else if (nprob .eq. 4) then
         a1 = 1.0d-3
         a2 = 1.0d-3
         t1 = sqrt(one+a1**2) - a1
         t2 = sqrt(one+a2**2) - a2
         f = t1*sqrt((one-x)**2+a2**2) + t2*sqrt(x**2+a1**2)
         g = -t1*((one-x)/sqrt((one-x)**2+a2**2)) +
     +       t2*(x/sqrt(x**2+a1**2))
      else if (nprob .eq. 5) then
         a1 = 1.0d-2
         a2 = 1.0d-3
         t1 = sqrt(one+a1**2) - a1
         t2 = sqrt(one+a2**2) - a2
         f = t1*sqrt((one-x)**2+a2**2) + t2*sqrt(x**2+a1**2)
         g = -t1*((one-x)/sqrt((one-x)**2+a2**2)) +
     +       t2*(x/sqrt(x**2+a1**2))
      else if (nprob .eq. 6) then
         a1 = 1.0d-3
         a2 = 1.0d-2
         t1 = sqrt(one+a1**2) - a1
         t2 = sqrt(one+a2**2) - a2
         f = t1*sqrt((one-x)**2+a2**2) + t2*sqrt(x**2+a1**2)
         g = -t1*((one-x)/sqrt((one-x)**2+a2**2)) +
     +       t2*(x/sqrt(x**2+a1**2))
      end if

      end
